

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.Paint;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;

import javax.swing.JFrame;

import edu.berkeley.guir.prefuse.Display;
import edu.berkeley.guir.prefuse.EdgeItem;
import edu.berkeley.guir.prefuse.ItemRegistry;
import edu.berkeley.guir.prefuse.NodeItem;
import edu.berkeley.guir.prefuse.VisualItem;
import edu.berkeley.guir.prefuse.action.AbstractAction;
import edu.berkeley.guir.prefuse.action.RepaintAction;
import edu.berkeley.guir.prefuse.action.assignment.ColorFunction;
import edu.berkeley.guir.prefuse.action.filter.GraphFilter;
import edu.berkeley.guir.prefuse.activity.ActionList;
import edu.berkeley.guir.prefuse.activity.Activity;
import edu.berkeley.guir.prefuse.graph.Graph;
import edu.berkeley.guir.prefuse.render.DefaultEdgeRenderer;
import edu.berkeley.guir.prefuse.render.DefaultNodeRenderer;
import edu.berkeley.guir.prefuse.render.DefaultRendererFactory;
import edu.berkeley.guir.prefuse.render.MultiLineTextItemRenderer;
import edu.berkeley.guir.prefuse.render.TextImageItemRenderer;
import edu.berkeley.guir.prefuse.render.TextItemRenderer;
import edu.berkeley.guir.prefusex.controls.DragControl;
import edu.berkeley.guir.prefusex.controls.FocusControl;
import edu.berkeley.guir.prefusex.controls.NeighborHighlightControl;
import edu.berkeley.guir.prefusex.controls.PanControl;
import edu.berkeley.guir.prefusex.controls.ZoomControl;
import edu.berkeley.guir.prefusex.force.DragForce;
import edu.berkeley.guir.prefusex.force.ForcePanel;
import edu.berkeley.guir.prefusex.force.ForceSimulator;
import edu.berkeley.guir.prefusex.force.GravitationalForce;
import edu.berkeley.guir.prefusex.force.NBodyForce;
import edu.berkeley.guir.prefusex.force.SpringForce;
import edu.berkeley.guir.prefusex.force.WallForce;
import edu.berkeley.guir.prefusex.layout.ForceDirectedLayout;
/**
 * Application demo of a graph visualization using an interactive
 * force-based layout.
 *
 * @version 1.0
 * @author <a href="http://jheer.org">Jeffrey Heer</a> prefuse(AT)jheer.org
 */
public class PhotoArcsDemo extends Display {

	private static int defaultX = 1000;
	private static int defaultY = 700;
	
    private JFrame     frame;
    private ForcePanel fpanel;
    
    private ForceSimulator m_fsim;
    private String         m_textField;
    private ItemRegistry   m_registry;
    private Activity       m_actionList;
    
    private Font frameCountFont = new Font("SansSerif", Font.PLAIN, 14);
    
    public PhotoArcsDemo(Graph g, ForceSimulator fsim) {
        this(g, fsim, "label");
    } //
    
    public PhotoArcsDemo(Graph g, ForceSimulator fsim, String textField) {
        // set up component first
        setSize(defaultX, defaultY);
        pan(defaultX/2, defaultY/2);
        fsim.addForce(new WallForce(-defaultX/2f, defaultY/2f, defaultX/2f, defaultY/2f));
        fsim.addForce(new WallForce(-defaultX/2f, -defaultY/2f, defaultX/2f, -defaultY/2f));
        fsim.addForce(new WallForce(defaultX/2f, -defaultY/2f, defaultX/2f, defaultY/2f));
        fsim.addForce(new WallForce(-defaultX/2f, -defaultY/2f, -defaultX/2f, defaultY/2f));
    	m_fsim = fsim;
        m_textField = textField;
        m_registry = new ItemRegistry(g);
        this.setItemRegistry(m_registry);
        initRenderers();
        m_actionList = initActionList();
        this.addControlListener(new NeighborHighlightControl());
        this.addControlListener(new DragControl(false, true));
        this.addControlListener(new FocusControl(0));
        this.addControlListener(new PanControl(false));
        this.addControlListener(new ZoomControl(false));
    } //
    
    public void runDemo() {
        // now set up application window
        fpanel = new ForcePanel(m_fsim);
        
        frame = new JFrame("PhotoArcs Demo");
        Container c = frame.getContentPane();
        c.setLayout(new BorderLayout());
        c.add(this, BorderLayout.CENTER);
        // this allows you to adjust the forces
        //c.add(fpanel, BorderLayout.EAST);
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        frame.addComponentListener(new ComponentAdapter() {
            public void componentResized(ComponentEvent e) {
                Dimension d = frame.getSize();
                Dimension p = fpanel.getSize();
                Insets in = frame.getInsets();
                PhotoArcsDemo.this.setSize(d.width-in.left-in.right-p.width,
                        d.height-in.top-in.bottom);
            } //
            
        });
        frame.pack();
        frame.setVisible(true);
        
        // start force simulation
        m_actionList.runNow();
    } //
    
    private void initRenderers() {
    	MultiLineTextItemRenderer textRenderer = new MultiLineTextItemRenderer();
    	TextImageItemRenderer nodeRenderer = new TextImageItemRenderer();
        nodeRenderer.setRenderType(TextItemRenderer.RENDER_TYPE_NONE);
        nodeRenderer.setMaxImageDimensions(100, 100);
        nodeRenderer.setImageSize(.5);
        //nodeRenderer.setHorizontalAlignment(0);
        nodeRenderer.setTextAttributeName(m_textField);
        DefaultEdgeRenderer edgeRenderer = new DefaultEdgeRenderer();    
        m_registry.setRendererFactory(new DefaultRendererFactory(
                nodeRenderer, edgeRenderer, null));
    } //
    
    private ActionList initActionList() {
    	ActionList setup = new ActionList(m_registry);
    	setup.add(new GraphFilter());
    	setup.add(new EndPointAction());
    	setup.runNow();
    	
        ActionList actionList = new ActionList(m_registry,-1,20);
    	actionList.add(new ResizeAction());
        actionList.add(new ForceDirectedLayout(m_fsim, false, false));
        actionList.add(new DemoColorFunction());
        actionList.add(new RepaintAction());
        return actionList;
    } //
    
    public static void main(String argv[]) {
    	// TODO take input; parse for different directories/arcs
        String[] defaultPhotoNames = new String[8];
        String[] defaultPhotoNames2 = new String[8];
        String defaultLocation = "http://sims.berkeley.edu/~morganya/images/";
        String defaultType = ".jpg";
        int sharedPhotoIndex = defaultPhotoNames2.length/2;
        int i = 0;
        int l = defaultPhotoNames.length;
        for (i=0; i<l; i++){
        	defaultPhotoNames[i] = defaultLocation + String.valueOf(i+1) + defaultType;
        } 
        for (i=l; i<l+defaultPhotoNames2.length; i++){
        	if (i == l+sharedPhotoIndex){
        		defaultPhotoNames2[i-l] = defaultLocation + String.valueOf(i-l+1) + defaultType;
        	} else if (i > l+defaultPhotoNames2.length/2 ) {
        		defaultPhotoNames2[i-l] = defaultLocation + String.valueOf(i) + defaultType;
        	} else {
        		defaultPhotoNames2[i-l] = defaultLocation + String.valueOf(i+1) + defaultType;
        	}
        } 
        String[][] photoArcs = new String[2][];
        photoArcs[0] = defaultPhotoNames;
        photoArcs[1] = defaultPhotoNames2;
        Graph g = PhotoArc.getArc(photoArcs);
        
        System.out.println("Visualizing PhotoSet: " + i + " photos");
        
        ForceSimulator fsim = new ForceSimulator();
        fsim.addForce(new GravitationalForce(.001f, 90));
        fsim.addForce(new NBodyForce(-0.4f, -1f, 0.9f));
        fsim.addForce(new SpringForce(1E-4f, 15f));
        fsim.addForce(new DragForce(-0.01f));
        
        PhotoArcsDemo fdemo = new PhotoArcsDemo(g, fsim);
        fdemo.runDemo();
    } //
    
    public static class ResizeAction extends AbstractAction {
    	public void run(ItemRegistry registry, double frac){
    		Iterator iter = registry.getNodeItems();
    		boolean first = true;
    		while ( iter.hasNext() ) {
    			NodeItem item = (NodeItem)iter.next();
    			if ( item instanceof NodeItem ) {
    				if ( item.isFocus() ) {
    					item.updateSize(2);
    					item.setSize(2);
    				} else {
    					item.updateSize(1);
    					item.setSize(1);
    				}
    			}
    		}
    	}
    }
    
    public static class EndPointAction extends AbstractAction {
     	public void run(ItemRegistry registry, double frac) {
    		Iterator iter = registry.getNodeItems();
    		double initialStartY1 = 0;
    		double initialEndY1 = 0;
    		while ( iter.hasNext() ) {
    			NodeItem item = (NodeItem)iter.next();
    			if ( item.getAttribute("identifier") == "start" ) {
    				// is first node
    				item.updateLocation(-defaultX/2, initialStartY1);
    				item.setLocation(-defaultX/2, initialStartY1);
    				item.setFixed(true);
    				// want them to cross if they have a node in common,
    				// but not cross otherwise
    				initialStartY1 = initialStartY1 + defaultY/4;
    			} else if ( item.getAttribute("identifier") == "end" ) {
    				// is last node
    				item.updateLocation(defaultX/2, initialEndY1);
    				item.setLocation(defaultX/2, initialEndY1);
    				item.setFixed(true);
    				// want them to cross if they have a node in common,
    				// but not cross otherwise
    				initialEndY1 = initialEndY1 - defaultY/4;
    			}
    		}
    	}
    }
    
    public class DemoColorFunction extends ColorFunction {
        private Color pastelRed = new Color(255,125,125);
        private Color pastelOrange = new Color(255,200,125);
        private Color lightGray = new Color(220,220,255);
        public Paint getColor(VisualItem item) {
            if ( item instanceof EdgeItem ) {
                if ( item.isHighlighted() )
                    return pastelOrange;
                else
                    return Color.LIGHT_GRAY;
            } else {
                return Color.BLACK;
            }
        } //
        public Paint getFillColor(VisualItem item) {
            if ( item.isHighlighted() )
                return pastelOrange;
            else if ( item instanceof NodeItem ) {
                if ( item.isFocus() )
                    return pastelRed;
                else
                    return lightGray;
            } else {
                return Color.BLACK;
            }
        } //        
    } //
    
} // end of class PhotoArcsDemo
